@@ -1,15 +1,75 @@
-import sys, os, re, subprocess, yaml, argparse, time, mimetypes
-
+import sys, os, re, subprocess, yaml, argparse, time, mimetypes, logging
+import typing as t
 from werkzeug.wrappers import Request, Response
 from werkzeug.middleware.shared_data import SharedDataMiddleware
 from werkzeug.middleware.http_proxy import ProxyMiddleware
 from werkzeug.middleware.dispatcher import DispatcherMiddleware
-from werkzeug.serving import run_simple
+from werkzeug.serving import run_simple, WSGIRequestHandler,_ansi_style,_log_add_style
 from werkzeug.wsgi import get_path_info, wrap_file
 from werkzeug.utils import get_content_type
 from werkzeug.http import http_date, is_resource_modified
+from werkzeug._internal import _logger
+from werkzeug.urls import uri_to_iri
+
+
+
+__version__ = "0.8.6"
+
+class myWSGIRequestHandler(WSGIRequestHandler):
+    def log_date_time_string(self):
+        """Return the current time formatted for logging."""
+        now = time.time()
+        year, month, day, hh, mm, ss, x, y, z = time.localtime(now)
+        s = "%04d-%02d-%02d %02d:%02d:%02d" % (
+            year , month,day , hh, mm, ss)
+        return s
+
+    def log_request(
+        self, code: t.Union[int, str] = "-", size: t.Union[int, str] = "-"
+    ) -> None:
+        """coloring the status code"""
+        try:
+            path = uri_to_iri(self.path)
+            msg = f"[{self.command}] {path}"
+        except AttributeError:
+            # path isn't set if the requestline was bad
+            msg = self.requestline
+
+        code = str(code)
+
+        log_type = "info"
+        if code != "200": #possibility to filter 200 requests
+            log_type = "warning"
+
+        if _log_add_style:
+            if code[0] == "1":  # 1xx - Informational
+                code = _ansi_style(code, "bold")
+            elif code == "200":  # 2xx - Success
+                pass
+            elif code == "304":  # 304 - Resource Not Modified
+                code = _ansi_style(code, "cyan")
+            elif code[0] == "3":  # 3xx - Redirection
+                code = _ansi_style(code, "green")
+            elif code == "404":  # 404 - Resource Not Found
+                code = _ansi_style(code, "yellow")
+            elif code[0] == "4":  # 4xx - Client Error
+                code = _ansi_style(code, "bold", "red")
+            else:  # 5xx, or any other response
+                code = _ansi_style(code, "bold", "red")
+
+        self.log(log_type, '[%s] %s', code, msg)
+
+    def log(self, type: str, message: str, *args) -> None:
+        global _logger
+
+        if _logger is None:
+            _logger = logging.getLogger("werkzeug")
+            _logger.setLevel(logging.INFO)
+            _logger.addHandler(logging.StreamHandler())
+
+        getattr(_logger, type)(f"[{self.log_date_time_string()}] {message % args}")
+
 
-__version__ = "0.7.3"
 
 
 class WrappingApp(object):
@@ -26,7 +86,6 @@ class WrappingApp(object):
     def __call__(self, environ, start_response):
         return self.wsgi_app(environ, start_response)
 
-
 class myProxy(ProxyMiddleware):
     """this addition allows to redirect all routes to given targets"""
 
@@ -171,10 +230,10 @@ def start_server(host, port, gunicorn_port, appFolder, appYaml, timeout, protoco
     app.wsgi_app = myDispatcher(app.wsgi_app, apps)
 
     time.sleep(5)
-    run_simple(host, port, app, use_debugger=False, use_reloader=True, threaded=True)
+    run_simple(host, port, app, use_debugger=False, use_reloader=True, threaded=True) #, request_handler=myWSGIRequestHandler
 
 
-def envVars(application_id):
+def envVars(application_id, args):
     """set nessesary environment variables"""
     os.environ["GAE_ENV"] = "localdev"
     os.environ["CLOUDSDK_CORE_PROJECT"] = application_id
@@ -182,6 +241,11 @@ def envVars(application_id):
     os.environ["GAE_VERSION"] = str(time.time())
     os.environ["GRPC_ENABLE_FORK_SUPPORT"] = "0"
 
+    if args.storage:
+        os.environ["STORAGE_EMULATOR_HOST"] = f"http://{args.host}:{args.storage_port}"
+
+    if args.tasks:
+        os.environ["TASKS_EMULATOR"] = f"{args.host}:{args.tasks_port}"
 
 def main():
     """main entrypoint
@@ -198,7 +262,7 @@ def main():
     ap.add_argument("config_paths", metavar='yaml_path', nargs='+', help='Path to app.yaml file')
     ap.add_argument(
         '-A', '--application', action='store', dest='app_id', required=True, help='Set the application id')
-    ap.add_argument('--host', default="localhost", help='host name to which application modules should bind')
+    ap.add_argument('--host', default="0.0.0.0", help='host name to which application modules should bind')
     ap.add_argument('--port', type=int, default=8080, help='port to which we bind the application')
     ap.add_argument('--gunicorn_port', type=int, default=8090, help='internal gunicorn port')
     ap.add_argument('--worker', type=int, default=1, help='amount of gunicorn workers')
@@ -206,8 +270,16 @@ def main():
     ap.add_argument('--timeout', type=int, default=60, help='Time is seconds befor gunicorn abort a rquest')
     ap.add_argument('-V', '--version', action='version', version='%(prog)s ' + __version__)
 
+    ap.add_argument('--storage', default=False, action="store_true", dest="storage", help="also start Storage Emulator")
+    ap.add_argument('--storage_port', type=int, default=8092, help='internal Storage Emulator Port')
+
+    ap.add_argument('--tasks', default=False, action='store_true', dest="tasks", help='also start Task-Queue Emulator')
+    ap.add_argument('--tasks_port', type=int, default=8091, help='internal Task-Queue Emulator Port')
+
+    ap.add_argument('--cron', default=False, action='store_true', dest="cron", help='also start Cron Emulator')
+
     args = ap.parse_args()
-    envVars(args.app_id)
+    envVars(args.app_id, args)
 
     myFolder = os.getcwd()
     appFolder = os.path.abspath(args.config_paths[0])
@@ -240,6 +312,19 @@ def main():
     os.chdir(appFolder)
     subprocess.Popen(entrypoint)
     os.chdir(myFolder)
+
+    if args.storage:
+        subprocess.Popen(
+            f"gcloud-storage-emulator start --port={args.storage_port} --default-bucket={args.app_id}.appspot.com".split())
+
+    if args.tasks and os.path.exists(os.path.join(appFolder, 'queue.yaml')):
+        cron = ""
+        if args.cron:
+            cron = f"--cron-yaml={os.path.join(appFolder, 'cron.yaml')}"
+
+        subprocess.Popen(
+            f"gcloud-tasks-emulator start -p={args.tasks_port} -t={args.port} {cron} --queue-yaml={os.path.join(appFolder, 'queue.yaml')} --queue-yaml-project={args.app_id} --queue-yaml-location=local -r 50".split())
+
     start_server(args.host, args.port, args.gunicorn_port, appFolder, appYaml, args.timeout)
 
 
